// ImpObject.cpp: implementation of the CImpObject class. 
// 
////////////////////////////////////////////////////////////////////// 
 
// visual c precompiled headers 
//#include "stdafx.h" 
 
// you may need something else here for sgi 
//#include <windows.h> 
#include <stdio.h>
#include "glut.h" 
 
// ansi headers 
#include <stdlib.h> 
#include <string.h> 
#include <math.h> 
 
// my object headers 
#include "ImpObject.h" 
 
 
 
 
 
#define RES 10 /* # converge iterations    */ 
 
#define DECOMP_MODE_TET 0  /* use tetrahedral decomposition */ 
#define DECOMP_MODE_NOTET 1  /* no tetrahedral decomposition  */ 
 
// Directions and corners for decomp 
#define L 0  /* left direction: -x, -i */ 
#define R 1  /* right direction: +x, +i */ 
#define B 2  /* bottom direction: -y, -j */ 
#define T 3  /* top direction: +y, +j */ 
#define N 4  /* near direction: -z, -k */ 
#define F 5  /* far direction: +z, +k */ 
 
#define LB 0  /* left bottom edge */ 
#define LT 1  /* left top edge */ 
#define LN 2  /* left near edge */ 
#define LF 3  /* left far edge */ 
#define RB 4  /* right bottom edge */ 
#define RT 5  /* right top edge */ 
#define RN 6  /* right near edge */ 
#define RF 7  /* right far edge */ 
#define BN 8  /* bottom near edge */ 
#define BF 9  /* bottom far edge */ 
#define TN 10 /* top near edge */ 
#define TF 11 /* top far edge */ 
 
#define LBN 0  /* left bottom near corner  */ 
#define LBF 1  /* left bottom far corner   */ 
#define LTN 2  /* left top near corner     */ 
#define LTF 3  /* left top far corner      */ 
#define RBN 4  /* right bottom near corner */ 
#define RBF 5  /* right bottom far corner  */ 
#define RTN 6  /* right top near corner    */ 
#define RTF 7  /* right top far corner     */ 
 
/* the LBN corner of cube (i, j, k), corresponds with location 
 * (start.x+(i-.5)*size, start.y+(j-.5)*size, start.z+(k-.5)*size) 
 */ 
 
#define RAND()     ((rand()&32767)/32767.)    /* random number between 0 and 1 */ 
 
////////////////////////////////////////////////////////////////////// 
// Objects static data 
////////////////////////////////////////////////////////////////////// 
/* edge: LB, LT, LN, LF, RB, RT, RN, RF, BN, BF, TN, TF */ 
int CImpObject::m_corner1[12]	= {LBN,LTN,LBN,LBF,RBN,RTN,RBN,RBF,LBN,LBF,LTN,LTF}; 
int CImpObject::m_corner2[12]	= {LBF,LTF,LTN,LTF,RBF,RTF,RTN,RTF,RBN,RBF,RTN,RTF}; 
int CImpObject::m_leftface[12]	= {B,  L,  L,  F,  R,  T,  N,  R,  N,  B,  T,  F}; /* face on left when going corner1 to corner2 */ 
int CImpObject::m_rightface[12]	= {L,  T,  N,  L,  B,  R,  R,  F,  B,  F,  N,  T}; /* face on right when going corner1 to corner2 */ 
 
INTLISTS *CImpObject::m_CubeTable[256]; 
 
 
 
 
 
////////////////////////////////////////////////////////////////////// 
// Construction/Destruction 
////////////////////////////////////////////////////////////////////// 
 
CImpObject::CImpObject() 
{ 
	// set default object data as per torus in demo 
	m_dSize=0.05;												// cube size 
	m_dDelta=m_dSize/(double)(RES*RES);							// normal delta 
	m_iBounds=20;												// cube range within lattice 
	m_Start.x=0; m_Start.y=0; m_Start.z=0;						// start point on surface 
	m_pCubes=NULL;												// active cubes 
	m_Vertices.count=m_Vertices.max=0; m_Vertices.ptr=NULL;		// surface vertices 
	m_pStashedTriangles=NULL; 
 
	// zero hash tables 
	memset( (void*)m_pCenters, 0, sizeof(CENTERLIST*)*HASHSIZE); 
	memset( (void*)m_pCorners, 0, sizeof(CORNERLIST*)*HASHSIZE); 
	memset( (void*)m_pEdges, 0, sizeof(EDGELIST*)*HASHSIZE*2); 
} 
 
CImpObject::~CImpObject() 
{ 
	// free mesh? 
	if( m_pStashedTriangles != NULL) 
		DestroyMesh(); 
} 
 
int CImpObject::CreateMesh(SurfaceFunc pSurfaceFunc, int ref, double size, int bounds, double x, double y, double z) 
{ 
	int		mode = DECOMP_MODE_NOTET; 
    int		n, noabort; 
    TEST	in, out; 
 
 
	// destroy old mesh? 
	if( m_pStashedTriangles != NULL) 
		DestroyMesh(); 
 
	m_pSurfaceFunc	= pSurfaceFunc; 
	m_iSurfaceRef	= ref; 
    m_dSize			= size; 
    m_iBounds		= bounds; 
    m_dDelta		= size/(double)(RES*RES); 
 
    /* find point on surface, beginning search at (x, y, z):  
	 */ 
    srand(1); 
    in = find(1, x, y, z); 
    out = find(0, x, y, z); 
    if (!in.ok || !out.ok) return -1; //"can't find starting point"; 
    converge(&in.p, &out.p, in.value, &m_Start, &m_Start);  //?? What vertex.color should be passed in here??? 
 
    /* push initial cube on stack 
	 */ 
    m_pCubes = (CUBES *) mycalloc(1, sizeof(CUBES)); /* list of 1 */ 
    m_pCubes->cube.i = m_pCubes->cube.j = m_pCubes->cube.k = 0; 
    m_pCubes->next = NULL; 
 
    /* set corners of initial cube 
	 */ 
    for (n = 0; n < 8; n++) 
		m_pCubes->cube.corners[n] = setcorner(BIT(n,2), BIT(n,1), BIT(n,0)); 
 
    m_Vertices.count = m_Vertices.max = 0; /* no vertices yet */ 
    m_Vertices.ptr = NULL; 
 
    setcenter( 0, 0, 0); 
 
    while (m_pCubes != NULL) 
	{ /* process active cubes till none left */ 
		CUBE	c;  
		CUBES	*temp = m_pCubes; 
 
 
		c = m_pCubes->cube; 
		noabort = mode == DECOMP_MODE_TET? 
 
		/* either decompose into tetrahedra and polygonize: */ 
		dotet(&c, LBN, LTN, RBN, LBF) && 
		dotet(&c, RTN, LTN, LBF, RBN) && 
		dotet(&c, RTN, LTN, LTF, LBF) && 
		dotet(&c, RTN, RBN, LBF, RBF) && 
		dotet(&c, RTN, LBF, LTF, RBF) && 
		dotet(&c, RTN, LTF, RTF, RBF) 
		: 
		/* or polygonize the cube directly: */ 
		docube(&c); 
 
		if (! noabort) return -1; //"aborted"; 
 
		/* pop current cube from stack */							/*WHAT IS THE CUBE STACK*/ 
		m_pCubes = m_pCubes->next; 
		free((char *) temp); 
		 
		/* test six face directions, maybe add to stack: */			/*NEEDED*/ 
		testface(c.i-1, c.j, c.k, &c, L, LBN, LBF, LTN, LTF);		/*WHY IS PROCESS PASSED???*/ 
		testface(c.i+1, c.j, c.k, &c, R, RBN, RBF, RTN, RTF); 
		testface(c.i, c.j-1, c.k, &c, B, LBN, LBF, RBN, RBF); 
		testface(c.i, c.j+1, c.k, &c, T, LTN, LTF, RTN, RTF); 
		testface(c.i, c.j, c.k-1, &c, N, LBN, LTN, RBN, RTN); 
		testface(c.i, c.j, c.k+1, &c, F, LBF, LTF, RBF, RTF); 
    } 
 
	return 0; 
} 
 
int CImpObject::DrawMesh() 
{ 
	STASHED_TRIANGLE	*pTri = m_pStashedTriangles; 
	bool havelow=false,havehigh=false; 
 
	glBegin(GL_TRIANGLES); 
 
	 
	while( pTri)  
	{  
		havelow=false,havehigh=false; 
	 	if (m_Vertices.ptr[pTri->v0].color.y<0.25) havelow=true; 
		if (m_Vertices.ptr[pTri->v1].color.y<0.25) havelow=true; 
		if (m_Vertices.ptr[pTri->v2].color.y<0.25) havelow=true; 
		if (m_Vertices.ptr[pTri->v0].color.y>0.75) havehigh=true; 
		if (m_Vertices.ptr[pTri->v1].color.y>0.75) havehigh=true; 
		if (m_Vertices.ptr[pTri->v2].color.y>0.75) havehigh=true; 
		  
 
		if ((havehigh) && (havelow)) 
		{	 
		glNormal3f(m_Vertices.ptr[pTri->v0].normal.x,  
					m_Vertices.ptr[pTri->v0].normal.y,  
					m_Vertices.ptr[pTri->v0].normal.z); 
		if  (m_Vertices.ptr[pTri->v0].color.y<0.25) 
			glTexCoord2f(m_Vertices.ptr[pTri->v0].color.x, m_Vertices.ptr[pTri->v0].color.y+1.0); 
		else  
			glTexCoord2f(m_Vertices.ptr[pTri->v0].color.x, m_Vertices.ptr[pTri->v0].color.y); 
		glVertex3f(m_Vertices.ptr[pTri->v0].position.x,  
					m_Vertices.ptr[pTri->v0].position.y,  
					m_Vertices.ptr[pTri->v0].position.z);  
				  
  
		glNormal3f(m_Vertices.ptr[pTri->v1].normal.x,  
					m_Vertices.ptr[pTri->v1].normal.y,  
					m_Vertices.ptr[pTri->v1].normal.z); 
		if  (m_Vertices.ptr[pTri->v1].color.y<0.25)  
			glTexCoord2f(m_Vertices.ptr[pTri->v1].color.x, m_Vertices.ptr[pTri->v1].color.y+1.0); 
		else 
	 		glTexCoord2f(m_Vertices.ptr[pTri->v1].color.x, m_Vertices.ptr[pTri->v1].color.y); 
		glVertex3f(m_Vertices.ptr[pTri->v1].position.x,  
   				   m_Vertices.ptr[pTri->v1].position.y,  
   				   m_Vertices.ptr[pTri->v1].position.z);  
  
		  
		glNormal3f(m_Vertices.ptr[pTri->v2].normal.x,  
					m_Vertices.ptr[pTri->v2].normal.y,  
					m_Vertices.ptr[pTri->v2].normal.z); 
		if  (m_Vertices.ptr[pTri->v2].color.y<0.25)  
			glTexCoord2f(m_Vertices.ptr[pTri->v2].color.x, m_Vertices.ptr[pTri->v2].color.y+1.0);  
		else 
			glTexCoord2f(m_Vertices.ptr[pTri->v2].color.x, m_Vertices.ptr[pTri->v2].color.y); 
		glVertex3f(m_Vertices.ptr[pTri->v2].position.x,  
					m_Vertices.ptr[pTri->v2].position.y,  
					m_Vertices.ptr[pTri->v2].position.z);  
		} 
 
 
		else 
		{	 
 		glNormal3f(m_Vertices.ptr[pTri->v0].normal.x,  
					m_Vertices.ptr[pTri->v0].normal.y,  
					m_Vertices.ptr[pTri->v0].normal.z);  
		glTexCoord2f(m_Vertices.ptr[pTri->v0].color.x, m_Vertices.ptr[pTri->v0].color.y);  
		glVertex3f(m_Vertices.ptr[pTri->v0].position.x,  
					m_Vertices.ptr[pTri->v0].position.y,  
					m_Vertices.ptr[pTri->v0].position.z);  
				  
  
		glNormal3f(m_Vertices.ptr[pTri->v1].normal.x,  
					m_Vertices.ptr[pTri->v1].normal.y,  
					m_Vertices.ptr[pTri->v1].normal.z);  
		glTexCoord2f(m_Vertices.ptr[pTri->v1].color.x, m_Vertices.ptr[pTri->v1].color.y);  
		glVertex3f(m_Vertices.ptr[pTri->v1].position.x,  
   				   m_Vertices.ptr[pTri->v1].position.y,  
   				   m_Vertices.ptr[pTri->v1].position.z);  
  
		  
		glNormal3f(m_Vertices.ptr[pTri->v2].normal.x,  
					m_Vertices.ptr[pTri->v2].normal.y,  
					m_Vertices.ptr[pTri->v2].normal.z);  
		glTexCoord2f(m_Vertices.ptr[pTri->v2].color.x, m_Vertices.ptr[pTri->v2].color.y);  
		glVertex3f(m_Vertices.ptr[pTri->v2].position.x,  
					m_Vertices.ptr[pTri->v2].position.y,  
					m_Vertices.ptr[pTri->v2].position.z);  
		} 
 
 
 
 
	      
  
  
		pTri = pTri->next;  
	}  
 
	 
 
	glEnd(); 
	 
	return 0; 
} 
 
int CImpObject::DestroyMesh() 
{ 
	int i; 
 
 
	// if no mesh then just return 
	if( m_pStashedTriangles == NULL) 
		return 0; 
 
	// free stashed tri list 
	while( m_pStashedTriangles) 
	{ 
		STASHED_TRIANGLE *pTri = m_pStashedTriangles; 
		m_pStashedTriangles = m_pStashedTriangles->next; 
		free( (void*)pTri); 
	} 
 
	// free cube list 
	while( m_pCubes) 
	{ 
		CUBES *pCube = m_pCubes; 
		m_pCubes = m_pCubes->next; 
 
		// free cube corners 
		for( i=0; i<8; i++) 
			if(	pCube->cube.corners[i]) 
				free( (void*)pCube->cube.corners[i]); 
 
		free( (void*)m_pCubes); 
	} 
 
	// free corner list 
	for( i=0; i<HASHSIZE; i++) 
		if( m_pCorners[i]) 
		{ 
			CORNERLIST *pCorners = m_pCorners[i]; 
 
			while( pCorners) 
			{ 
				CORNERLIST *pTmp = pCorners; 
				pCorners = pCorners->next; 
				free( (void*)pTmp); 
			} 
		} 
 
	// free centers 
	for( i=0; i<HASHSIZE; i++) 
		if( m_pCenters[i]) 
		{ 
			CENTERLIST *pCenters = m_pCenters[i]; 
 
			while( pCenters) 
			{ 
				CENTERLIST *pTmp = pCenters; 
				pCenters = pCenters->next; 
				free( (void*)pTmp); 
			} 
		} 
 
	// free edges 
	for( i=0; i<(HASHSIZE*2); i++) 
		if( m_pEdges[i]) 
		{ 
			EDGELIST *pEdges = m_pEdges[i]; 
 
			while( pEdges) 
			{ 
				EDGELIST *pTmp = pEdges; 
				pEdges = pEdges->next; 
				free( (void*)pTmp); 
			} 
		} 
 
	// zero hash tables 
	memset( (void*)m_pCenters, 0, sizeof(CENTERLIST*)*HASHSIZE); 
	memset( (void*)m_pCorners, 0, sizeof(CORNERLIST*)*HASHSIZE); 
	memset( (void*)m_pEdges, 0, sizeof(EDGELIST*)*HASHSIZE*2); 
 
	// free vertices 
	free( (void*)m_Vertices.ptr); 
 
	return 0; 
} 
 
/**** Cubical Polygonization (optional) ****/ 
 
/* makecubetable: create the 256 entry table for cubical polygonization 
 */ 
void CImpObject::MakeCubeTable() 
{ 
	int	i, e, c, done[12], pos[8]; 
 
 
	for (i = 0; i < 256; i++) 
	{ 
		for (e = 0; e < 12; e++) 
			done[e] = 0; 
 
		for (c = 0; c < 8; c++) 
			pos[c] = BIT(i, c); 
 
		for (e = 0; e < 12; e++) 
			if (!done[e] && (pos[m_corner1[e]] != pos[m_corner2[e]])) 
			{ 
				INTLIST *ints = 0; 
				INTLISTS *lists = (INTLISTS *) mycalloc(1, sizeof(INTLISTS)); //////////////////////////////////////////////// 
				int start = e, edge = e; 
 
				/* get face that is to right of edge from pos to neg corner: */ 
				int face = pos[m_corner1[e]]? m_rightface[e] : m_leftface[e]; 
 
				while (1) 
				{ 
					edge = NextCWEdge(edge, face); 
					done[edge] = 1; 
 
					if (pos[m_corner1[edge]] != pos[m_corner2[edge]]) 
					{ 
						INTLIST *tmp = ints; 
 
						ints = (INTLIST *) mycalloc(1, sizeof(INTLIST)); ////////////////////////////////////////////////////// 
						ints->i = edge; 
						ints->next = tmp; /* add edge to head of list */ 
 
						if (edge == start) 
							break; 
 
						face = OtherFace(edge, face); 
					} 
				} 
 
				lists->list = ints; /* add ints to head of table entry */ 
				lists->next = m_CubeTable[i]; 
				m_CubeTable[i] = lists; 
			} 
	} 
} 
 
/* nextcwedge: return next clockwise edge from given edge around given face 
 */ 
int CImpObject::NextCWEdge(int edge, int face) 
{ 
    switch (edge) 
	{ 
		case LB: return (face == L)? LF : BN; 
		case LT: return (face == L)? LN : TF; 
		case LN: return (face == L)? LB : TN; 
		case LF: return (face == L)? LT : BF; 
		case RB: return (face == R)? RN : BF; 
		case RT: return (face == R)? RF : TN; 
		case RN: return (face == R)? RT : BN; 
		case RF: return (face == R)? RB : TF; 
		case BN: return (face == B)? RB : LN; 
		case BF: return (face == B)? LB : RF; 
		case TN: return (face == T)? LT : RN; 
		case TF: return (face == T)? RT : LF; 
    } 
 
	// default 
	return LF; 
} 
 
/* otherface: return face adjoining edge that is not the given face 
 */ 
int CImpObject::OtherFace(int edge, int face) 
{ 
    int other = m_leftface[edge]; 
    return face == other? m_rightface[edge] : other; 
} 
 
void CImpObject::Init() 
{ 
	MakeCubeTable(); 
} 
 
char * CImpObject::mycalloc(int nitems, int nbytes) 
{ 
   char *ptr = (char*)calloc(nitems, nbytes); 
   if (ptr != NULL) return ptr; 
   fprintf(stderr, "can't calloc %d bytes\n", nitems*nbytes); 
   exit(1); 
} 
 
/* find: search for point with value of given sign (0: neg, 1: pos) */ 
TEST CImpObject::find(int sign, double x, double y, double z) 
{ 
	int		i; 
	TEST	test; 
	double	range = m_dSize; 
	impValues  vals; 
 
 
	test.ok = 1; 
	for (i = 0; i < 10000; i++) 
	{ 
		test.p.x = x+range*(RAND()-0.5); 
		test.p.y = y+range*(RAND()-0.5); 
		test.p.z = z+range*(RAND()-0.5); 
		vals = m_pSurfaceFunc( m_iSurfaceRef, test.p.x, test.p.y, test.p.z); 
		test.value = vals.density; 
 
		if (sign == (test.value > 0.0)) return test; 
			range = range*1.0005; /* slowly expand search outwards */ 
	} 
	test.ok = 0; 
	return test; 
} 
 
/* converge: from two points of differing sign, converge to zero crossing */ 
void CImpObject::converge(POINT3 *p1, POINT3 *p2, double v, POINT3 *p, POINT3 *c) 
{ 
	int		i = 0; 
	POINT3	pos, neg; 
	impValues vals; 
 
 
	if (v < 0) 
	{ 
		pos.x = p2->x; pos.y = p2->y; pos.z = p2->z; 
		neg.x = p1->x; neg.y = p1->y; neg.z = p1->z; 
	} 
	else 
	{ 
		pos.x = p1->x; pos.y = p1->y; pos.z = p1->z; 
		neg.x = p2->x; neg.y = p2->y; neg.z = p2->z; 
	} 
 
	while (1) 
	{ 
		//position and color calculation 
		p->x = 0.5*(pos.x + neg.x); 
		p->y = 0.5*(pos.y + neg.y); 
		p->z = 0.5*(pos.z + neg.z); 
		if (i++ == RES) return; 
		vals = m_pSurfaceFunc( m_iSurfaceRef, p->x, p->y, p->z); 
			if (vals.density > 0.0) 
		{ 
			pos.x = p->x; pos.y = p->y; pos.z = p->z; 
		} 
		else 
		{ 
			neg.x = p->x; neg.y = p->y; neg.z = p->z; 
		} 
 
		c->x = vals.color.x; 
		c->y = vals.color.y; 
		c->z = vals.color.z; 
	} 
} 
 
/* setcorner: return corner with the given lattice location 
   set (and cache) its function value 
 */ 
CORNER * CImpObject::setcorner(int i, int j, int k) 
{ 
	/* for speed, do corner value caching here 
	 */ 
	CORNER		*c = (CORNER *) mycalloc(1, sizeof(CORNER)); 
	int			index = HASH(i, j, k); 
	CORNERLIST	*l = m_pCorners[index]; 
	impValues vals; 
 
	c->i = i; c->x = m_Start.x+((double)i-0.5)*m_dSize; 
	c->j = j; c->y = m_Start.y+((double)j-0.5)*m_dSize; 
	c->k = k; c->z = m_Start.z+((double)k-0.5)*m_dSize; 
 
	for (; l != NULL; l = l->next) 
		if (l->i == i && l->j == j && l->k == k) 
		{ 
			c->value = l->value; 
			return c; 
		} 
 
	l = (CORNERLIST *) mycalloc(1, sizeof(CORNERLIST)); 
	l->i = i; l->j = j; l->k = k; 
	vals = m_pSurfaceFunc( m_iSurfaceRef, c->x, c->y, c->z); 
	l->value = c->value = vals.density; 
	l->next = m_pCorners[index]; 
	m_pCorners[index] = l; 
	return c; 
} 
 
/* setcenter: set (i,j,k) entry of table[] 
 * return 1 if already set; otherwise, set and return 0 
 */ 
int CImpObject::setcenter(int i, int j, int k) 
{ 
	int			index = HASH(i, j, k); 
	CENTERLIST	*new_center, *l, *q = m_pCenters[index]; 
 
 
	for (l = q; l != NULL; l = l->next) 
		if (l->i == i && l->j == j && l->k == k) return 1; 
 
	new_center = (CENTERLIST *) mycalloc(1, sizeof(CENTERLIST)); 
	new_center->i = i; new_center->j = j; new_center->k = k; 
	new_center->next = q; 
	m_pCenters[index] = new_center; 
	return 0; 
} 
 
/**** Tetrahedral Polygonization ****/ 
/* dotet: triangulate the tetrahedron 
 * b, c, d should appear clockwise when viewed from a 
 * return 0 if client aborts, 1 otherwise 
 */ 
int CImpObject::dotet(CUBE *cube, int c1, int c2, int c3, int c4) 
{ 
	CORNER *a = cube->corners[c1]; 
	CORNER *b = cube->corners[c2]; 
	CORNER *c = cube->corners[c3]; 
	CORNER *d = cube->corners[c4]; 
	int index = 0, apos, bpos, cpos, dpos, e1, e2, e3, e4, e5, e6; 
 
 
	if (apos = (a->value > 0.0)) index += 8; 
	if (bpos = (b->value > 0.0)) index += 4; 
	if (cpos = (c->value > 0.0)) index += 2; 
	if (dpos = (d->value > 0.0)) index += 1; 
 
	/* index is now 4-bit number representing one of the 16 possible cases*/ 
	if (apos != bpos) e1 = vertid(a, b); 
	if (apos != cpos) e2 = vertid(a, c); 
	if (apos != dpos) e3 = vertid(a, d); 
	if (bpos != cpos) e4 = vertid(b, c); 
	if (bpos != dpos) e5 = vertid(b, d); 
	if (cpos != dpos) e6 = vertid(c, d); 
 
	/* 14 productive tetrahedral cases (0000 and 1111 do not yield polygons*/ 
	switch (index) 
	{ 
		case 1:  return triangle(e5, e6, e3);		/*TRIPROC CALLED*/ 
		case 2:  return triangle(e2, e6, e4); 
		case 3:  return triangle(e3, e5, e4) && 
						triangle(e3, e4, e2); 
		case 4:  return triangle(e1, e4, e5); 
		case 5:  return triangle(e3, e1, e4) && 
						triangle(e3, e4, e6); 
		case 6:  return triangle(e1, e2, e6) && 
						triangle(e1, e6, e5); 
		case 7:  return triangle(e1, e2, e3); 
		case 8:  return triangle(e1, e3, e2); 
		case 9:  return triangle(e1, e5, e6) && 
						triangle(e1, e6, e2); 
		case 10: return triangle(e1, e3, e6) && 
						triangle(e1, e6, e4); 
		case 11: return triangle(e1, e5, e4); 
		case 12: return triangle(e3, e2, e4) && 
						triangle(e3, e4, e5); 
		case 13: return triangle(e6, e2, e4); 
		case 14: return triangle(e5, e3, e6); 
	} 
	return 1; 
} 
 
/**** Vertices ****/ 
 
/* vertid: return index for vertex on edge: 
 * c1->value and c2->value are presumed of different sign 
 * return saved index if any; else compute vertex and save 
 */ 
int CImpObject::vertid(CORNER *c1, CORNER *c2) 
{ 
	VERTEX	v; 
	POINT3	a, b; 
	int vid = getedge( c1->i, c1->j, c1->k, c2->i, c2->j, c2->k); 
 
 
	if (vid != -1) return vid;      /* previously computed */ 
	a.x = c1->x; a.y = c1->y; a.z = c1->z; 
	b.x = c2->x; b.y = c2->y; b.z = c2->z; 
	converge(&a, &b, c1->value, &v.position, &v.color); /* position & COLOR*/ 
	vnormal(&v.position, &v.normal);    /* normal */ 
	addtovertices( v);    /* save vertex */ 
	vid = m_Vertices.count-1; 
	setedge( c1->i, c1->j, c1->k, c2->i, c2->j, c2->k, vid); 
	return vid; 
} 
 
/* addtovertices: add v to sequence of vertices */ 
void CImpObject::addtovertices( VERTEX v) 
{ 
	if (m_Vertices.count == m_Vertices.max) 
	{ 
		int i; 
		VERTEX *new_vertex; 
 
 
		m_Vertices.max = m_Vertices.count == 0 ? 10 : 2*m_Vertices.count; 
		new_vertex = (VERTEX *) mycalloc(m_Vertices.max, sizeof(VERTEX)); 
 
		for (i = 0; i < m_Vertices.count; i++) new_vertex[i] = m_Vertices.ptr[i]; 
		if (m_Vertices.ptr != NULL) 
			free((char *) m_Vertices.ptr); 
 
		m_Vertices.ptr = new_vertex; 
	} 
	m_Vertices.ptr[m_Vertices.count++] = v; 
} 
 
/* vnormal: compute unit length surface normal at point */ 
void CImpObject::vnormal(POINT3 *point, POINT3 *v) 
{ 
	impValues vals; 
	vals = m_pSurfaceFunc( m_iSurfaceRef, point->x, point->y, point->z); 
	double f = vals.density; 
 
	vals = m_pSurfaceFunc( m_iSurfaceRef, point->x+m_dDelta, point->y, point->z); 
	v->x = vals.density - f; 
	vals = m_pSurfaceFunc( m_iSurfaceRef, point->x, point->y+m_dDelta, point->z); 
	v->y = vals.density - f; 
	vals = m_pSurfaceFunc( m_iSurfaceRef, point->x, point->y, point->z+m_dDelta); 
	v->z = vals.density - f; 
	f = sqrt(v->x*v->x + v->y*v->y + v->z*v->z); 
	if (f != 0.0) 
	{ 
		v->x /= f; v->y /= f; v->z /= f; 
	} 
} 
 
/* setedge: set vertex id for edge */ 
void CImpObject::setedge( int i1, int j1, int k1, int i2, int j2, int k2, int vid) 
{ 
	unsigned int index; 
	EDGELIST *new_edge; 
 
 
	if (i1>i2 || (i1==i2 && (j1>j2 || (j1==j2 && k1>k2)))) 
	{ 
		int t=i1; 
		i1=i2; i2=t; t=j1; j1=j2; j2=t; t=k1; k1=k2; k2=t; 
	} 
 
	index = HASH(i1, j1, k1) + HASH(i2, j2, k2); 
	new_edge = (EDGELIST *) mycalloc(1, sizeof(EDGELIST)); 
	new_edge->i1 = i1; new_edge->j1 = j1; new_edge->k1 = k1; 
	new_edge->i2 = i2; new_edge->j2 = j2; new_edge->k2 = k2; 
	new_edge->vid = vid; 
	new_edge->next = m_pEdges[index]; 
	m_pEdges[index] = new_edge; 
} 
 
/* getedge: return vertex id for edge; return -1 if not set */ 
int CImpObject::getedge( int i1, int j1, int k1, int i2, int j2, int k2) 
{ 
	EDGELIST *q; 
 
 
	if (i1>i2 || (i1==i2 && (j1>j2 || (j1==j2 && k1>k2)))) 
	{ 
		int t=i1; 
		i1=i2; i2=t; t=j1; j1=j2; j2=t; t=k1; k1=k2; k2=t; 
	} 
 
	q = m_pEdges[HASH(i1, j1, k1)+HASH(i2, j2, k2)]; 
	for (; q != NULL; q = q->next) 
		if (q->i1 == i1 && q->j1 == j1 && q->k1 == k1 && 
			q->i2 == i2 && q->j2 == j2 && q->k2 == k2) 
			return q->vid; 
	return -1; 
} 
 
/* triangle: called by polygonize() for each triangle; write to stdout */ 
/* DRAWS GL TRIANGLES */ 
int CImpObject::triangle(int i1, int i2, int i3) 
{ 
	STASHED_TRIANGLE *temp = (STASHED_TRIANGLE*)mycalloc( 1, sizeof(STASHED_TRIANGLE)); 
 
	temp->next = m_pStashedTriangles; 
	m_pStashedTriangles = temp; 
 
	temp->v0 = i1; 
	temp->v1 = i2; 
	temp->v2 = i3; 
 
	return 1; 
} 
 
/* docube: triangulate the cube directly, without decomposition */ 
int CImpObject::docube(CUBE *cube) 
{ 
	INTLISTS *polys; 
	int i, index = 0; 
 
 
	for (i = 0; i < 8; i++) 
		if (cube->corners[i]->value > 0.0) 
			index += (1<<i); 
 
	for (polys = m_CubeTable[index]; polys; polys = polys->next) 
	{ 
		INTLIST *edges; 
		int a = -1, b = -1, count = 0; 
 
		for (edges = polys->list; edges; edges = edges->next) 
		{ 
			CORNER *c1 = cube->corners[m_corner1[edges->i]]; 
			CORNER *c2 = cube->corners[m_corner2[edges->i]]; 
			int c = vertid(c1, c2); 
 
			if (++count > 2 && ! triangle(a, b, c)) return 0; 
			if (count < 3) a = b; 
			b = c; 
		} 
	} 
	return 1; 
} 
 
/* testface: given cube at lattice (i, j, k), and four corners of face, 
 * if surface crosses face, compute other four corners of adjacent cube 
 * and add new cube to cube stack 
 */ 
void CImpObject::testface(int i, int j, int k, CUBE *old, int face, int c1, int c2, int c3, int c4) 
{ 
	CUBE		new_cube; 
	CUBES		*oldcubes = m_pCubes; 
	static int	facebit[6] = {2, 2, 1, 1, 0, 0}; 
	int			n, pos = old->corners[c1]->value > 0.0 ? 1 : 0, bit = facebit[face]; 
 
	/* test if no surface crossing, cube out of bounds, or already visited:*/ 
	if ((old->corners[c2]->value > 0) == pos && 
		(old->corners[c3]->value > 0) == pos && 
		(old->corners[c4]->value > 0) == pos) 
		return; 
 
	if (abs(i) > m_iBounds || abs(j) > m_iBounds || abs(k) > m_iBounds) 
		return; 
 
	if (setcenter( i, j, k)) 
		return; 
 
	/* create new cube: */ 
	new_cube.i = i; 
	new_cube.j = j; 
	new_cube.k = k; 
	for (n = 0; n < 8; n++) 
		new_cube.corners[n] = NULL; 
 
	new_cube.corners[FLIP(c1, bit)] = old->corners[c1]; 
	new_cube.corners[FLIP(c2, bit)] = old->corners[c2]; 
	new_cube.corners[FLIP(c3, bit)] = old->corners[c3]; 
	new_cube.corners[FLIP(c4, bit)] = old->corners[c4]; 
 
	for (n = 0; n < 8; n++) 
		if (new_cube.corners[n] == NULL) 
			new_cube.corners[n] = setcorner(i+BIT(n,2), j+BIT(n,1), k+BIT(n,0)); 
 
	/*add cube to top of stack: */ 
	m_pCubes = (CUBES *) mycalloc(1, sizeof(CUBES)); 
	m_pCubes->cube = new_cube; 
	m_pCubes->next = oldcubes; 
} 
