// ImpObject.h: interface for the CImpObject class. 
// 
////////////////////////////////////////////////////////////////////// 
 
#if !defined(AFX_IMPOBJECT_H__3A52FC60_E4CE_11D4_9784_004095455DAE__INCLUDED_) 
#define AFX_IMPOBJECT_H__3A52FC60_E4CE_11D4_9784_004095455DAE__INCLUDED_ 
 
#if _MSC_VER > 1000 
#pragma once 
#endif // _MSC_VER > 1000 
 
 
 
struct aPoint 
{ 
	float x,y,z; 
}; 
 
 
struct impValues 
{ 
	float density; 
	aPoint color; 
}; 
 
// prototype for implicit surface function 
typedef impValues (*SurfaceFunc)( int ref, double x, double y, double z); 
 
// Hash table stuff 
#define HASHBIT     (5) 
#define HASHSIZE    (size_t)(1<<(3*HASHBIT)) 
#define MASK		((1<<HASHBIT)-1) 
#define HASH(i,j,k) ((((((i)&MASK)<<HASHBIT)|((j)&MASK))<<HASHBIT)|((k)&MASK)) 
#define BIT(i, bit) (((i)>>(bit))&1) 
#define FLIP(i,bit) ((i)^1<<(bit)) /* flip the given bit of i */ 
 
// Structures 
typedef struct point  /* a three-dimensional point */ 
{ 
    double x, y, z;    /* its coordinates */ 
} POINT3; 
 
typedef struct test 
{    /* test the function for a signed value */ 
    POINT3 p;    /* location of test */ 
    double value;    /* function value at p */ 
    int ok;    /* if value is of correct sign */ 
} TEST; 
 
typedef struct vertex 
{    /* surface vertex */ 
    POINT3 position, normal, color;    /* position and surface normal */ 
} VERTEX; 
 
typedef struct vertices 
{    /* list of vertices in polygonization */ 
    int count, max;    /* # vertices, max # allowed */ 
    VERTEX *ptr;    /* dynamically allocated */ 
} VERTICES; 
 
typedef struct corner 
{    /* corner of a cube */ 
    int i, j, k;    /* (i, j, k) is index within lattice */ 
    double x, y, z, value;    /* location and function value */ 
} CORNER; 
 
typedef struct cube 
{    /* partitioning cell (cube) */ 
    int i, j, k;    /* lattice location of cube */ 
    CORNER *corners[8];    /* eight corners */ 
} CUBE; 
 
typedef struct cubes 
{    /* linked list of cubes acting as stack */ 
    CUBE cube;    /* a single cube */ 
    struct cubes *next;    /* remaining elements */ 
} CUBES; 
 
typedef struct centerlist 
{    /* list of cube locations */ 
    int i, j, k;    /* cube location */ 
    struct centerlist *next;    /* remaining elements */ 
} CENTERLIST; 
 
typedef struct cornerlist 
{    /* list of corners */ 
    int i, j, k;    /* corner id */ 
    double value;    /* corner value */ 
    struct cornerlist *next;    /* remaining elements */ 
} CORNERLIST; 
 
typedef struct edgelist 
{    /* list of edges */ 
    int i1, j1, k1, i2, j2, k2;    /* edge corner ids */ 
    int vid;    /* vertex id */ 
    struct edgelist *next;    /* remaining elements */ 
} EDGELIST; 
 
typedef struct intlist 
{    /* list of integers */ 
    int i;    /* an integer */ 
    struct intlist *next;    /* remaining elements */ 
} INTLIST; 
 
typedef struct intlists 
{    /* list of list of integers */ 
    INTLIST *list;    /* a list of integers */ 
    struct intlists *next;    /* remaining elements */ 
} INTLISTS; 
 
 
// gl triangle info 
typedef struct STASHED_TRIANGLE_TAG 
{ 
	int						v0;		// 3 vertex indices 
	int						v1; 
	int						v2; 
	STASHED_TRIANGLE_TAG	*next; 
} STASHED_TRIANGLE; 
 
 
 
class CImpObject   
{ 
public: 
	static void Init( void); 
	int DestroyMesh( void); 
	int DrawMesh( void); 
	int CreateMesh( SurfaceFunc pSurfaceFunc, int ref, double size, int bounds, double x, double y, double z); 
	CImpObject(); 
	virtual ~CImpObject(); 
 
private: 
	void testface( int i, int j, int k, CUBE *old, int face, int c1, int c2, int c3, int c4); 
	int docube( CUBE *cube); 
	int triangle( int i1, int i2, int i3); 
	int getedge( int i1, int j1, int k1, int i2, int j2, int k2); 
	void setedge( int i1, int j1, int k1, int i2, int j2, int k2, int vid); 
	void vnormal( POINT3 *point, POINT3 *v); 
	void addtovertices( VERTEX v); 
	int vertid( CORNER *c1, CORNER *c2); 
	int dotet( CUBE *cube, int c1, int c2, int c3, int c4); 
	int setcenter( int i, int j, int k); 
	CORNER * setcorner( int i, int j, int k); 
	void converge( POINT3 *p1, POINT3 *p2, double v, POINT3 *p, POINT3 *c); 
	TEST find( int sign, double x, double y, double z); 
	static char * mycalloc( int nitems, int nbytes); 
	static int OtherFace( int edge, int face); 
	static int NextCWEdge( int edge, int face); 
	static void MakeCubeTable( void); 
 
	static	INTLISTS	*m_CubeTable[256]; 
 
	SurfaceFunc	m_pSurfaceFunc;				// func describing surface 
	int			m_iSurfaceRef;				// ref to pass back to surface func to identify surface 
 
	double		m_dSize;					// cube size 
	double		m_dDelta;					// normal delta 
	int			m_iBounds;					// cube range within lattice 
	POINT3		m_Start;					// start point on surface 
	CUBES		*m_pCubes;					// active cubes 
	VERTICES	m_Vertices;					// surface vertices 
	CENTERLIST	*m_pCenters[HASHSIZE];		// cube center hash table 
	CORNERLIST	*m_pCorners[HASHSIZE];		// corner value hash table 
	EDGELIST	*m_pEdges[HASHSIZE*2];		// edge and vertex id hash table 
 
	STASHED_TRIANGLE	*m_pStashedTriangles;	// list of triangles generated by polygoniser 
 
	static int m_corner1[12]; 
	static int m_corner2[12]; 
	static int m_leftface[12];	// face on left when going corner1 to corner2 
	static int m_rightface[12];	// face on right when going corner1 to corner2 
 
}; 
 
#endif // !defined(AFX_IMPOBJECT_H__3A52FC60_E4CE_11D4_9784_004095455DAE__INCLUDED_) 
