/******************************************************************************/ 
/* *.OBJ LOADER CODE by ROB BATEMAN   Created:       10/May/2000              */ 
/*                                    Ammended:      25/May/2000              */ 
/******************************************************************************/ 
 
#//include <stdafx.h>  
#include <stdio.h> 
#include <stdlib.h>  
#include <string.h>  
#include <assert.h> 
#include <ctype.h> 
#include "glut.h" 
#include "Obj.h" 
//#include "dib.h"  
  
/*====== Function to create polygons from vertices held in a text file =======*/  
/*============================================================================*/ 
 
/* Parameters are *.obj filename, texture array, display list number          */ 
/* and then the materials colour in RED, GREEN, BLUE and ALPHA.		      */ 
//edited by dfletcher - removed RGBA values  
int LoadObjData(char *objname,  int texnum, GLuint display_list_number) 
{  
   FILE *ObjData;			  
   int av,avt,avn,bv,bvt,bvn,cv,cvt,cvn,dv,dvt,dvn,i,texturing_on=0,surfaces=0;  
   float x_coord = 0, y_coord = 0, z_coord = 0;  
   char line[256] = {0},temp[6];  
   char *SPtr;  
    
 
/* Create the head and tail pointers for each queue */ 
 
   VERTEXPTR headPtr_V = NULL, tailPtr_V = NULL;   
   FACEPTR headPtr_F = NULL, tailPtr_F = NULL;  
   NORMALPTR headPtr_N = NULL, tailPtr_N = NULL;   
   TEXVERTEXPTR headPtr_T = NULL, tailPtr_T = NULL;   
 
 
/* open the .obj file and load the data into queues */ 
  
  if((ObjData = fopen(objname, "r")) == NULL)  
  {  
    printf("\nERROR: Unable to open level data file.\n");  
    return 0;  
  }  
  else  
  {  
    while(!feof(ObjData))  
    {  
 
      for(i=0;i<255;i++) 
        line[i] = ' '; /* ensures nothing is left in string before copying new one */ 
 
      /* Grab a Line of the *.obj file */  
      fgets(line,256,ObjData);  
  
      /* compare the first 2 characters of the line. if 'vn' vertex normal, 'vt' texture coordinate, 	*/ 
      /* 'v ' a vertex, or if 'f ' then its a face. each has its own unique id number			*/ 
 
      if (strncmp(line,"o ",2) == 0) 
      { 
        line[0] = ' '; 
        printf("OBJECT NAME:%s\n",line); 
      } 
      else if (strncmp(line,"usemtl",6) == 0) 
      {  
	  sscanf(line,"%s%d",temp,&i); 
	  surfaces = i; 
      }  
      else if (strncmp(line,"vn",2) == 0) 
      {  
        sscanf(line,"%s%f%f%f",temp,&x_coord,&y_coord,&z_coord);  
        enqueue_normal(&headPtr_N,&tailPtr_N,x_coord, y_coord, z_coord);  
      }  
      else if (strncmp(line,"vt",2) == 0) 
      { 
        texturing_on = 1; 
        sscanf(line,"%s%f%f%f",temp,&x_coord,&y_coord,&z_coord);  
        enqueue_texture_vertex(&headPtr_T,&tailPtr_T,x_coord, y_coord, z_coord);  
      }  
      else if (strncmp(line,"v ",2) == 0) 
      {  
        sscanf(line,"%s%f%f%f",temp,&x_coord,&y_coord,&z_coord);   
        enqueue_vertex(&headPtr_V,&tailPtr_V,x_coord, y_coord, z_coord);  
      }  
      else if(strncmp(line,"f ",2) == 0) 
      { 
        for(i=0;i<256;i++) 
          /* replaces horrible slashes with spaces */ 
          if(!isdigit(line[i])) 
            line[i]= ' '; 
 
        /* reset the 4th vertex to zero, incase a triangle appears */ 
        dv=0; 
        dvt=0; 
        dvn=0; 
 
        /* read off the id numbers from line. If a triangle is read, the fourth vertex id will still be zero */ 
        if (texturing_on == 1 ) 
        { 
          sscanf(line,"%d%d%d%d%d%d%d%d%d%d%d%d",&av,&avt,&avn, 
                                                 &bv,&bvt,&bvn, 
                                                 &cv,&cvt,&cvn, 
                                                 &dv,&dvt,&dvn); 
 
          /* wang the data into a face */ 
          enqueue_face(&headPtr_F, &tailPtr_F,headPtr_V,headPtr_T,headPtr_N, 
                       av, avt, avn, bv, bvt, bvn, cv, cvt, cvn, dv, dvt, dvn, surfaces); 
        } 
        else  
        { 
          sscanf(line,"%d%d%d%d%d%d%d%d",&av,&avn, 
                                         &bv,&bvn, 
                                         &cv,&cvn, 
                                         &dv,&dvn); 
 
          /* wang the data into a face */ 
          enqueue_face(&headPtr_F, &tailPtr_F,headPtr_V,headPtr_T,headPtr_N, 
                       av, 0, avn, bv, 0, bvn, cv, 0, cvn, dv, 0, dvn, surfaces); 
        }      
      } 
    } 
    fclose(ObjData);		/* Close The File */  
 
 
 
 
    /* Draw, and sort out, the colour and texture for this object */ 
 
        /* Assigns colour for the drawn object */ 
        //glColor4f(R,G,B,A); //removed  - dfletcher 
         
		/* calls function to draw the object */ 
    		draw_Obj(headPtr_F,texnum,display_list_number); 
 
 
 
 
    /* kills all dynamically stored data */ 
    while(headPtr_F != NULL) 
      dequeue_face(&headPtr_F,&tailPtr_F); 
    while(headPtr_V != NULL) 
      dequeue_vertex(&headPtr_V,&tailPtr_V);  		  
    while(headPtr_T != NULL) 
      dequeue_texture_vertex(&headPtr_T,&tailPtr_T); 
    while(headPtr_N != NULL) 
      dequeue_normal(&headPtr_N,&tailPtr_N); 
 
  }  
  return 1; 
}  
/*=========================== Adds A New Node ================================*/  
/*============================================================================*/  
  
void enqueue_vertex(VERTEXPTR *HeadPtr, VERTEXPTR *TailPtr, float x, float y, float z)  
{  
  VERTEXPTR newPtr=NULL;  
  
  newPtr = (ObjVERTEX *)malloc(sizeof(ObjVERTEX));  
  
  if (newPtr != NULL) 
  {		   
    newPtr->x = x;		   
    newPtr->y = y;  
    newPtr->z = z;  
 
    if(*TailPtr == NULL) 
      newPtr->ID_NUM = 1; 
    else 
      newPtr->ID_NUM = (*TailPtr)->ID_NUM + 1;	 
    newPtr->nextPtr = NULL;    
  
    if (*HeadPtr==NULL)  
      *HeadPtr = newPtr;	  
    else  
      (*TailPtr)->nextPtr = newPtr; 
    *TailPtr = newPtr;  
  }    
}  
  
void enqueue_normal(NORMALPTR *HeadPtr, NORMALPTR *TailPtr, float x, float y, float z)  
{  
  NORMALPTR newPtr=NULL;  
  
  newPtr = (NORMAL *)malloc(sizeof(NORMAL));  
  
  if (newPtr != NULL)  
  {		   
    newPtr->x = x;		   
    newPtr->y = y;  
    newPtr->z = z;  
 
    if(*TailPtr == NULL) 
      newPtr->ID_NUM = 1; 
    else  
      newPtr->ID_NUM = (*TailPtr)->ID_NUM +1;		 
    newPtr->nextPtr = NULL;    
  
    if (*HeadPtr==NULL)  
      *HeadPtr = newPtr;	  
    else  
      (*TailPtr)->nextPtr = newPtr;  
    *TailPtr = newPtr;  
  }    
}  
  
void enqueue_texture_vertex(TEXVERTEXPTR *HeadPtr, TEXVERTEXPTR *TailPtr, float x, float y, float z)  
{  
  TEXVERTEXPTR newPtr=NULL;  
  
  newPtr = (TEXVERTEX *)malloc(sizeof(TEXVERTEX));  
  
  if (newPtr != NULL) {		   
    newPtr->x = x;		   
    newPtr->y = y;  
    newPtr->z = z; 
 
    if(*TailPtr == NULL)  
      newPtr->ID_NUM = 1;  
    else 
      newPtr->ID_NUM = (*TailPtr)->ID_NUM +1; 
    newPtr->nextPtr = NULL;    
  
    if (*HeadPtr==NULL)  
      *HeadPtr = newPtr;	  
    else  
      (*TailPtr)->nextPtr = newPtr;  
    *TailPtr = newPtr;  
  }    
}  
  
  
void enqueue_face(FACEPTR *HeadPtr, FACEPTR *TailPtr, 
                  VERTEXPTR headPtr_V,  
                  TEXVERTEXPTR headPtr_T, 
                  NORMALPTR headPtr_N,  
                  int av, int avt, int avn, 
                  int bv, int bvt, int bvn, 
                  int cv, int cvt, int cvn, 
                  int dv, int dvt, int dvn,int surface)  
{  
	 
	FACEPTR newPtr = NULL; 
 
	newPtr = (FACE *)malloc(sizeof(FACE)); 
  
	/* all pass_id functions return a pointer to the relevant queue node */ 
	if (newPtr != NULL) {	   
	  newPtr->va = pass_id(av,headPtr_V);		   
	  newPtr->ta = pass_idt(avt,headPtr_T);  
	  newPtr->na = pass_idn(avn,headPtr_N);  
	  newPtr->vb = pass_id(bv,headPtr_V);		   
	  newPtr->tb = pass_idt(bvt,headPtr_T); 
	  newPtr->nb = pass_idn(bvn,headPtr_N); 
	  newPtr->vc = pass_id(cv,headPtr_V);		   
	  newPtr->tc = pass_idt(cvt,headPtr_T);  
	  newPtr->nc = pass_idn(cvn,headPtr_N); 
	  newPtr->vd = pass_id(dv,headPtr_V);		   
	  newPtr->td = pass_idt(dvt,headPtr_T);  
	  newPtr->nd = pass_idn(dvn,headPtr_N); 
	  newPtr->texture = surface;   /* ensure it knows which texture is applied to which face */ 
 
	  if(*TailPtr == NULL)  
	    newPtr->IDNUM = 1; 
	  else  
	    newPtr->IDNUM = (*TailPtr)->IDNUM +1; 
  
	  newPtr->nextPtr = NULL; 
    
	  if (*HeadPtr==NULL)  
	    *HeadPtr = newPtr;	  
	  else  
          (*TailPtr)->nextPtr = newPtr;  
	  *TailPtr = newPtr;  
 	}     
 	else printf("malloc failed\n");  
 
}  
  
/*================== Functions to dequeue stored vertices etc ================*/  
/*============================================================================*/  
  
void dequeue_vertex(VERTEXPTR *HeadPtr, VERTEXPTR *TailPtr)  
{ 
  VERTEXPTR tempPtr; 
  tempPtr = *HeadPtr; 
  *HeadPtr = (*HeadPtr)->nextPtr; 
  if (*HeadPtr == NULL)  *TailPtr = NULL; 
  free(tempPtr); 
}  
  
void dequeue_texture_vertex(TEXVERTEXPTR *HeadPtr, TEXVERTEXPTR *TailPtr)  
{  
  TEXVERTEXPTR tempPtr; 
  tempPtr = *HeadPtr; 
  *HeadPtr = (*HeadPtr)->nextPtr; 
  if (*HeadPtr == NULL)  *TailPtr = NULL; 
  free(tempPtr); 
}  
  
void dequeue_normal(NORMALPTR *HeadPtr, NORMALPTR *TailPtr)  
{ 
  NORMALPTR tempPtr; 
  tempPtr = *HeadPtr; 
  *HeadPtr = (*HeadPtr)->nextPtr; 
  if (*HeadPtr == NULL)  *TailPtr = NULL; 
  free(tempPtr);     
}  
  
void dequeue_face(FACEPTR *HeadPtr, FACEPTR *TailPtr)  
{  
  FACEPTR tempPtr; 			  
  tempPtr = *HeadPtr;  
  *HeadPtr = (*HeadPtr)->nextPtr;		  
  if (*HeadPtr == NULL)  *TailPtr = NULL; 
  free(tempPtr); 
}  
  
/*==========Func to take id_num of vertex and return a pointer to it==========*/  
/*============================================================================*/  
  
VERTEXPTR pass_id (int i,VERTEXPTR headPtr_V)  
{  
  VERTEXPTR currentPtr=headPtr_V;  
  while (currentPtr!=NULL && currentPtr->ID_NUM != i && currentPtr!=0) 
    currentPtr= currentPtr->nextPtr;  
 
  if (i==0) currentPtr =  NULL; 
  return currentPtr;  
}  
  
  
/*=========Func to take id_num of tex coord and return a pointer to it========*/  
/*============================================================================*/  
  
TEXVERTEXPTR pass_idt (int i,TEXVERTEXPTR headPtr_T)  
{  
  TEXVERTEXPTR currentPtr=headPtr_T;  
  while (currentPtr!=NULL && currentPtr->ID_NUM != i&& currentPtr!=0)   
    currentPtr= currentPtr->nextPtr;  
  
  if (i==0) currentPtr =  NULL; 
  return currentPtr;  
}  
  
/*======Func to take id_num of vertex normal and return a pointer to it=======*/  
/*============================================================================*/  
  
NORMALPTR pass_idn (int i, NORMALPTR headPtr_N)  
{  
  NORMALPTR currentPtr=headPtr_N;  
  while (currentPtr!=NULL && currentPtr->ID_NUM != i&& currentPtr!=0)   
    currentPtr= currentPtr->nextPtr;  
 
  if (i==0) currentPtr =  NULL; 
  return currentPtr; 
}  
 
/*================= Function to load a texture hassle free ===================*/  
/*============================================================================*/ 
 
//GLuint load_texture ( char *texturefile ) 
//{ 
   //not needed - use tga.cpp instead - removed by dfletcher 
//} 
 
  
/*========================= Function to draw Object =========================*/  
/*============================================================================*/  
  
static void draw_Obj(FACEPTR SPtr,int texnum,GLuint disp)  
{  
FACEPTR currentPtr = SPtr; /* create pointer to traverse the face queue */ 
int currentTexture; 
 
if (currentPtr->ta == NULL) 
{ 
//This used to be for an obj with no texture - removed by dfletcher 
	printf("I shouldn't be doing this no texture thing\n"); 
} 
else 
{ 
      glNewList(disp,GL_COMPILE); 
      glPushMatrix(); 
	  glEnable(GL_TEXTURE_2D); 
	  glBindTexture(GL_TEXTURE_2D, texnum);	//TO DO 
	while(currentPtr != NULL) 
	{ 
	 if (currentPtr->vd==NULL) 
	  {  
          glBegin(GL_TRIANGLES);  
	      glTexCoord2f((currentPtr->ta)->x,(currentPtr->ta)->y); 
	      glNormal3f((currentPtr->na)->x,(currentPtr->na)->y,(currentPtr->na)->z); 
            glVertex3f((currentPtr->va)->x,(currentPtr->va)->y,(currentPtr->va)->z); 
	      glTexCoord2f((currentPtr->tb)->x,(currentPtr->tb)->y);  
	      glNormal3f((currentPtr->nb)->x,(currentPtr->nb)->y,(currentPtr->nb)->z); 
            glVertex3f((currentPtr->vb)->x,(currentPtr->vb)->y,(currentPtr->vb)->z); 
	      glTexCoord2f((currentPtr->tc)->x,(currentPtr->tc)->y); 
	      glNormal3f((currentPtr->nc)->x,(currentPtr->nc)->y,(currentPtr->nc)->z); 
            glVertex3f((currentPtr->vc)->x,(currentPtr->vc)->y,(currentPtr->vc)->z); 
          glEnd();  
      }  
     else  
	  {  
          glBegin(GL_QUADS);  
	      glTexCoord2f((currentPtr->ta)->x,(currentPtr->ta)->y); 
	      glNormal3f  ((currentPtr->na)->x,(currentPtr->na)->y,(currentPtr->na)->z); 
            glVertex3f  ((currentPtr->va)->x,(currentPtr->va)->y,(currentPtr->va)->z);  
	      glTexCoord2f((currentPtr->tb)->x,(currentPtr->tb)->y); 
	      glNormal3f  ((currentPtr->nb)->x,(currentPtr->nb)->y,(currentPtr->nb)->z); 
            glVertex3f  ((currentPtr->vb)->x,(currentPtr->vb)->y,(currentPtr->vb)->z); 
	      glTexCoord2f((currentPtr->tc)->x,(currentPtr->tc)->y); 
	      glNormal3f  ((currentPtr->nc)->x,(currentPtr->nc)->y,(currentPtr->nc)->z); 
            glVertex3f  ((currentPtr->vc)->x,(currentPtr->vc)->y,(currentPtr->vc)->z);  
	      glTexCoord2f((currentPtr->td)->x,(currentPtr->td)->y); 
            glNormal3f  ((currentPtr->nd)->x,(currentPtr->nd)->y,(currentPtr->nd)->z); 
            glVertex3f  ((currentPtr->vd)->x,(currentPtr->vd)->y,(currentPtr->vd)->z); 
	    glEnd(); 
	  } 
	  currentPtr = currentPtr->nextPtr; 
	} 
	glEnable(GL_TEXTURE_2D); 
      glPopMatrix(); 
      glEndList(); 
} 
 
 
}  
