/*PLANETS. An opengl game by David Fletcher 11/2000*/

/* 
	This game demonstrates the use of an implicit surface polygonizer 
	within a computer game environment. The Polygonizer was written by James Bloomenthal in 'c'.
	It was updated by James Fletcher to make use of c++ object orientated programming.

*/
 
 
//WINDOWS INCLUDES
 
#include <stdafx.h> 
#include <windows.h> 
#include <GL\glut.h> 
#include <stdlib.h> 
#include <time.h> 
#include <math.h> 
#include <string.h> 
#include <assert.h> 
#include "ImpObject.h" 
#include "tga.h" 
#include "font.h"
#include "Obj.h" 

 
//SGI 
 /*
#include <stdio.h> 
#include "glut.h" 
#include <stdlib.h> 
#include <time.h> 
#include <math.h> 
#include <string.h> 
#include <assert.h> 
#include "ImpObject.h" 
#include "Tga.h" 
#include "font.h"
#include "Obj.h"
 */
 
 
#define PI 3.141592654 
#define TESTSIZE 0.1 
#define pspeedMIN 0.08	//minimum selectable speed of projectile
#define pspeedMAX 0.2   //maximum selectable speed of projectile
#define NUMPLANETS 5	//number of planets to generate - have enough textures for 8
#define MAXPLANETSIZE 3	
#define STARS 2000		//number of stars to be created for background
						//		- displayed as GL points
 
 
/* Portability stuff... */ 
#ifdef WIN32   
	typedef __int64 int64; 
#else 
  	typedef long int64; 
 
#endif 
 
 
 
 
bool oktogo1 = true, oktogo2 = true, help=true; 
bool keys[128], specials[128], firing, fired, lmb, mmb , newGame = true, newGame2 = true, asknewGame = false; 
int mouse_press_x, mouse_press_y, turnEnd, player=0, notplayer=1, ExplodeTime=1,camera=1,p1score=0,p2score=0,turnlength=0; 
int EndTurnState = 2;	// 0 = hit ship 0 
						// 1 = hit ship 1 
						// 2 = hit nothing 
						// 3 = hit planet 
GLfloat spin_x, spin_y, dolly, trans_x, trans_y; //Camera movement variables 
static int turn=0; //Initial turn set to player 1
 
float  craterSize=0.5,CUBESIZE=0.3,numcubes=(MAXPLANETSIZE*2)/CUBESIZE+1;
// - crater size - how big a crater should be - influenced by power
// - CUBESIZE - how big polygonizing cubes should be - affects detail of planets
//			  - 0.3 is good for fast PC, 0.8ish is good for SGI
// - numcubes - max number of cubes used to polygonize planet - affected by CUBESIZE and max planet size
 
int winW = 640; /* window width */ 
int winH = 480; /* window height */ 

GLuint object[2];	//Used to store ship display list IDs     
 
//a sphere for use as a planets hole linked list
struct aSphere			
{ 
	float radius; 
	aPoint position; 
	aSphere *nextPtr; 
}; 
 
aSphere *currentSphere; 
 

struct aPlanet 
{ 
	float x,y,z,size; 
	CImpObject	*PlanetObj; 
	aSphere *spheres;		//linked list for holes
}; 
 
aPlanet planets[NUMPLANETS+1]; 
 
 
struct aTail //Smoke trail strcuture 
{ 
	aPoint pos; 
	aTail *nextPtr,*prevPtr; 
	float fade; 
}; 
 
 
 
struct aShip 
{ 
	float x,y,z,size,px,py,pz,pxold,pyold,pzold,	//size - used for collision detection
			rx,ry,rz,vx,vy,vz,pspeed; 
 
}; 
 
 
aPoint space[STARS],explodePos;
 
aShip Ships[2]; 
 
aTail *tail; 
 



 
/* Function prototypes =======================================================*/ 
int initFrametime(void); 
double ftime(void); 
void sortFrametime(void); 
int DrawUniverse(void); 
int StartGame(void); 
int fireGun(void); 
int animateGun(void); 
void BuildUniverse(void); 
void KillUniverse(void); 
void DrawHud(void); 
void explode(int); 
void DrawProjectile(void); 
void killTail(void); 
void initTextures(void); 
void planetDebris(void);
 
/* Globals ===================================================================*/ 
/* Stuff for keeping track of frametime... */ 
int64 startclock;   /* Start time */ 
double rate_inv, frametime, framestart, frameend; 
 
 
 
/* Functions =================================================================*/ 
 
/* Frame counter - initialise ================================================*/ 
int initFrametime(void) 
{ 
   rate_inv = 1.0/(double)CLOCKS_PER_SEC; 
   return 1; 
} 
 
 
 
/* Frame counter - get the timer's value and divide by its rate ==============*/ 
double ftime(void) 
{ 
   return clock()*rate_inv; 
} 
 
 
 
/* Frame counter - work out current frametime ================================*/ 
void sortFrametime(void) 
{ 
   do{ 
      frameend=ftime(); 
   }while(frameend==framestart); 
 
   frametime=frameend-framestart; 
   framestart=frameend; 
 
   
  /* frametime = 0.05;   use this line on non windows os */ 
} 
 
 
 
 
 
////////////////////////////////////////////////////////////////////////// 
// implicit surface functions 
 
 
/* p is point tested, c is centre point */ 
double dsphere( double px, double py, double pz, double cx, double cy, double cz, double radius ) 
{ 
return (px - cx)*(px - cx) + (py - cy)*(py - cy) + (pz - cz)*(pz - cz) - radius*radius; 
} 
 
 
/* pSurface: a planet with craters in*/ 
impValues pSurface ( int ref, double x, double y, double z) 
{ 
	impValues vals; 
	float radius; 
	//printf("%d", ref); 
	currentSphere = planets[ref].spheres; 
	double strength2; 
	vals.density = dsphere( x, y, z, 0, 0, 0, planets[ref].size ); 
	 
 	//SPHERICAL MAPPING 
	vals.color.y = atan2(z,x)/(2*PI); if (vals.color.y < 0) vals.color.y = vals.color.y + 1.0; 
	radius = planets[ref].size; 
	vals.color.x = ((asin(y/radius))/PI + 0.5)/1.25 + 0.2 ; 
  
	vals.color.z = 0.0; 
 
	while (currentSphere != NULL) 
	{ 
		strength2 = -1*(dsphere( x, y, z, currentSphere->position.x ,  
										  currentSphere->position.y ,  
										  currentSphere->position.z ,  
										  currentSphere->radius       )); 
 
		if (vals.density<strength2)  
		{ 
				vals.density = strength2; 
				//SPHERICAL MAPPING 
				vals.color.y = atan2(z,x)/(2*PI); if (vals.color.y < 0) vals.color.y = vals.color.y + 1.0; 
				radius = planets[ref].size; 
				vals.color.x = ( (asin(y/radius))/PI + 0.5 ) / 10; 
 
		} 
		currentSphere = currentSphere->nextPtr; 
	} 
	return vals; 
} 
////////////////////////////////////////////////////////////////////////////// 
 
 
 
 
 
 
void init(void) 
{ 
	glClearColor(0.0, 0.0, 0.0, 0.0); 
	glShadeModel (GL_SMOOTH);
	glEnable(GL_BLEND);
	glEnable(GL_CULL_FACE);
	glCullFace(GL_FRONT);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	GLfloat light_position[] = { 10.0, 10.0, 10.0, 0.0 }; 
	glLightfv(GL_LIGHT0, GL_POSITION, light_position); 
	glEnable(GL_LIGHTING); 
	glEnable(GL_LIGHT0); 
	glEnable(GL_DEPTH_TEST); 
	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
	
	//LOAD TEXTURES
	loadTGA ("textures/texture.tga",  13); 
	loadTGA ("textures/texture2.tga", 14); 
	loadTGA ("textures/texture3.tga", 15); 
	loadTGA ("textures/texture4.tga", 16); 
	loadTGA ("textures/texture5.tga", 17); 
	loadTGA ("textures/texture6.tga", 18); 
	loadTGA ("textures/texture7.tga", 19); 
	loadTGA ("textures/texture8.tga", 20);
	loadTGA ("textures/Ship3.tga",    41);
	loadTGA ("textures/Ship4.tga",    40);
	loadTGA ("textures/universe3.tga", 50);
	
	//LOAD SHIP OBJECTS
	object[0] = glGenLists(1);
	LoadObjData("objects/ship0.obj", 40, object[0]); 
	object[1] = glGenLists(1);
	LoadObjData("objects/ship1.obj", 41, object[1]);
 

	 if(!initFrametime()) 
      { 
         printf("ERROR: Unable to initialise frame counter.\n\n"); 
         exit(0); 
      } 
	
	 makeRasterFont();	//INITIALISE FONTS
	 
 
} 
 
void display(void) 
{ 
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
 
	glPushMatrix (); 
	 
if (camera == 1)	//Standard camera 
{ 
   glPushMatrix(); 
    glTranslatef(trans_x, trans_y, dolly); 
    glRotatef(spin_x, 1.0, 0.0, 0.0); 
    glPushMatrix(); 
     glRotatef(spin_y, 0.0, 1.0, 0.0); 
     glPushMatrix(); 
      gluLookAt(0, 0, 20, 0, 0, 0, 0, 1, 0); 
	 DrawUniverse(); 
	 glPopMatrix(); 
    glPopMatrix(); 
   glPopMatrix(); 
} 

 
if (camera == 5)	//Universal Camera - unmovable
{ 
   glPushMatrix(); 
    glPushMatrix(); 
      glPushMatrix(); 
      gluLookAt(0, 0, 20, 0, 0, 0, 0, 1, 0); 
	  DrawUniverse(); 
	 glPopMatrix(); 
    glPopMatrix(); 
   glPopMatrix(); 
} 

 
if (camera == 2)	//Ship 0 POV 
{ 
   glPushMatrix(); 
    glRotatef(Ships[0].rx, 1.0, 0.0, 0.0); 
    glPushMatrix(); 
     glRotatef(-1*Ships[0].ry, 0.0, 1.0, 0.0); 
     glPushMatrix(); 
	  gluLookAt(-10, 0, 0, 0, 0, 0, 0, 1, 0); 
	  DrawUniverse(); 
	 glPopMatrix(); 
    glPopMatrix(); 
   glPopMatrix(); 
} 
 
if (camera == 3)	//Ship 1 POV 
{ 
   glPushMatrix(); 
    
    glRotatef(Ships[1].rx, 1.0, 0.0, 0.0); 
    glPushMatrix(); 
     glRotatef(-1*Ships[1].ry+180, 0.0, 1.0, 0.0); 
     glPushMatrix(); 
	  gluLookAt( 10, 0, 0, 0, 0, 0, 0, 1, 0); 
	  DrawUniverse(); 
	 glPopMatrix(); 
    glPopMatrix(); 
   glPopMatrix(); 
} 
 
if (camera == 4)	//Projectile POV 
{ 
   glPushMatrix(); 
   	  gluLookAt(Ships[player].pxold, Ships[player].pyold, Ships[player].pzold, 
				Ships[player].px, Ships[player].py, Ships[player].pz, 
				0.0, 1.0, 0.0 ); 
	  DrawUniverse(); 
	glPopMatrix(); 
} 
 
		 
glPopMatrix(); 
	 
//HUD DRAWING
glPushMatrix (); 
	glOrtho(0.0,100.0,0.0,100.0,0.0,100.0); 
	glShadeModel (GL_SMOOTH);	 
	glDisable(GL_LIGHTING); 
	glDisable(GL_LIGHT0); 
	DrawHud(); 
glPopMatrix(); 
	 
 
glutSwapBuffers(); 
 
 
} 
 
void reshape (int w, int h) 
{  
	glViewport(0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_PROJECTION); 
	glLoadIdentity(); 
	gluPerspective(60.0, (GLfloat) w/(GLfloat) h, 0.07, 500.0); 
	glMatrixMode(GL_MODELVIEW); 
	glLoadIdentity(); 
} 
 
 
//******** Keyboard UP functions ********* 
void specialUp (int key, int x, int y) 
{ 
	specials[key] = false; 
} 
 
void keyUp (unsigned char key, int x, int y) 
{ 
 
	keys[key] = false; 
} 
 
 
 
//******** Keyboard DOWN functions ********* 
 
void specialDown (int key, int x, int y) 
{ 
	specials[key] = true; 
} 
 
void keyDown (unsigned char key, int x, int y) 
{ 
	keys[key] = true; 
} 
 
 

 
/* Mouse input function ======================================================*/ 
void mouse(int button, int state, int x, int y) 
{ 
   if(state == GLUT_DOWN) 
   { 
      mouse_press_x = x; mouse_press_y = y; 
 
      switch(button) 
      { 
         case GLUT_LEFT_BUTTON: 
            lmb = true; 
            break; 
 
         case GLUT_MIDDLE_BUTTON: 
            mmb = true; 
            break; 
 
         default: 
            break; 
      } 
   } 
 
   if(state == GLUT_UP) 
   { 
      glutSetCursor(GLUT_CURSOR_NONE); 
      switch(button) 
      { 
         case GLUT_LEFT_BUTTON: 
            lmb = false; 
            break; 
 
         case GLUT_MIDDLE_BUTTON: 
            mmb = false; 
            break; 
 
         default: 
            break; 
      } 
   } 
 
   glutPostRedisplay(); 
} 
 
/* Motion function ===========================================================*/ 
void motion(int x, int y) 
{ 

	if (camera==1)
	{
   int step_x = (x - mouse_press_x), step_y = (y - mouse_press_y); 
 
   mouse_press_x = x; mouse_press_y = y; 
 
   if(lmb && !mmb)        /* Tumble */ 
   { 
#ifdef WIN32 
      glutSetCursor(GLUT_CURSOR_CROSSHAIR); 
#else 
      glutSetCursor(GLUT_CURSOR_CYCLE); 
#endif 
      spin_x += 0.5*step_y; 
      spin_y += 0.5*step_x; 
   } 
   else if(mmb && !lmb)   /* Translate */ 
   { 
#ifdef WIN32 
      glutSetCursor(GLUT_CURSOR_SPRAY); 
#else 
      glutSetCursor(GLUT_CURSOR_CROSSHAIR); 
#endif 
      trans_x += 0.03*step_x; 
      trans_y -= 0.03*step_y; 
   } 
   else if(lmb && mmb) /* Dolly */ 
	   
   { 
      glutSetCursor(GLUT_CURSOR_LEFT_RIGHT); 
      dolly += 0.03*step_x; 
	  if (dolly > 50) dolly = 50;			//boundary positive
	  if (dolly < -50) dolly = -50;			//boundary negative
   } 
   glutPostRedisplay(); 
	}


} 
 


 
void idle(void) 
{ 
	
	if(newGame)
	{
	init(); 
	StartGame();
	newGame  = false;
	newGame2 = false;
	}
 
	if (!asknewGame) if ( !fired )if (keys[' ']) { firing = true; newGame2 = true; }  
 
	if (keys['='] )  if (Ships[player].pspeed < pspeedMAX) if (oktogo1)  
		{ Ships[player].pspeed = Ships[player].pspeed + 0.02;  oktogo1 = false; } 
	if (keys['='] ) oktogo1 = false; else oktogo1 = true ; 
	if (keys['-'] )  if (Ships[player].pspeed > pspeedMIN) if (oktogo2)  
		{ Ships[player].pspeed = Ships[player].pspeed - 0.02;  oktogo2 = false; } 
	if (keys['-'] ) oktogo2 = false; else oktogo2 = true ; 
	
	craterSize = Ships[player].pspeed*4; //adjust crater size
 
	if (keys['1']) camera = 1; 
	if (keys['2']) camera = 2; 
	if (keys['3']) camera = 3; 
	if (keys['4']) { if(fired) camera = 4; else camera=player+2; }
	if (keys['5']) camera = 5;
 
	if (keys['q']) fired = false;  //kill missile - cheat

	if (keys['h']) help = false;  //close help screen

	if (asknewGame) if (newGame2) if (keys['n']) { newGame = true; asknewGame = false; }
 
	 
	//ROTATE SHIP
		if (specials[GLUT_KEY_LEFT]) 
		{ 
			(Ships[player].ry)++; 
		} 
 
		if (specials[GLUT_KEY_RIGHT]) 
		{ 
			(Ships[player].ry)--; 
		} 
 
		if (specials[GLUT_KEY_UP])  
		{ 
			(Ships[player].rx)++; 
		} 
 
		if (specials[GLUT_KEY_DOWN])  
		{ 
			(Ships[player].rx)--; 
		} 
	 
 
	 
	//EXIT GAME
	if (keys[27]) 
	{ 
		KillUniverse();
		if(glutGameModeGet(GLUT_GAME_MODE_ACTIVE)) glutLeaveGameMode();
		exit(0); 
	} 
	 
 	glutPostRedisplay(); 
 
} 
 
 
 
int DrawUniverse(void) 
{	 
		 
		//Draw Space 
		glEnable (GL_TEXTURE_2D); /* enable texture mapping */ 
		glBindTexture (GL_TEXTURE_2D, 50); /* bind to a texture*/ 
		glPushMatrix(); 
			glCullFace(GL_BACK);
			glColor3f(1.0,1.0,1.0); 
			planets[NUMPLANETS].PlanetObj->DrawMesh();
			glCullFace(GL_FRONT);
			glDisable (GL_TEXTURE_2D); /* disable texture mapping */  
		glPopMatrix();
		
		
	//DRAW STARS - GL POINTS
	glPushMatrix();
	glDisable(GL_LIGHTING); 
	glDisable(GL_LIGHT0);
	for(int j=0;j<STARS;j++)
	{
		glBegin(GL_POINTS);
		glColor3f(1.0,1.0,1.0);
		glVertex3f(space[j].x,space[j].y,space[j].z);
		glEnd();
	}
	glEnable(GL_LIGHTING); 
	glEnable(GL_LIGHT0); 
	glPopMatrix();



		//Draw Ships 
		for (int i=0;i<2;i++) 
		{ 
			glPushMatrix(); 
				glCullFace(GL_BACK);
				glTranslatef(Ships[i].x, Ships[i].y, Ships[i].z); 
				glRotatef(90, 0.0, 1.0, 0.0 ); 
				glRotatef(Ships[i].ry, 0.0, 1.0, 0.0 ); 
				glRotatef(Ships[i].rx, 1.0, 0.0, 0.0 ); 
				glCallList(object[i]);	//DRAW CURRENT SHIP OBJECT
				glCullFace(GL_FRONT);
			glPopMatrix(); 
		} 


		//Draw player point-to arrow

		glDisable(GL_LIGHTING); 
		glDisable(GL_LIGHT0);
		//glPolygonMode (GL_BACK, GL_LINE);
		glCullFace(GL_BACK);
		glDisable (GL_TEXTURE_2D);
		if (player==0)
		{
			glPushMatrix();
				glTranslatef(Ships[0].x, Ships[0].y, Ships[0].z);
				glTranslatef(0.0,2.5,0.0);
				glRotatef(90,1.0,0.0,0.0);
				glColor4f(0.0,1.0,0.0,0.8);
				glutSolidCone(0.5, 1.0, 10,1);
			glPopMatrix();
		}

		if (player==1)
		{
			glPushMatrix();
				glTranslatef(Ships[1].x, Ships[1].y, Ships[1].z);
				glTranslatef(0.0,2.5,0.0);
				glRotatef(90,1.0,0.0,0.0);
				glColor4f(0.0,1.0,0.0,0.8);
				glutSolidCone(0.5, 1.0, 10,1);
			glPopMatrix();
		}
		glEnable(GL_LIGHTING); 
		glEnable(GL_LIGHT0); 
		glCullFace(GL_FRONT);
		//glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);

 
		//Draw Planets 
		for(i=0;i<NUMPLANETS;i++) 
		{ 
			glPushMatrix(); 
				glTranslatef(planets[i].x, planets[i].y, planets[i].z); 
				glEnable (GL_TEXTURE_2D); /* enable texture mapping */ 
				glBindTexture (GL_TEXTURE_2D, 13+i); /* bind to a texture*/ 
				planets[i].PlanetObj->DrawMesh(); 
				glDisable (GL_TEXTURE_2D); /* disable texture mapping */
			glPopMatrix(); 
		} 
	

	if (!asknewGame)
	{
		if (firing) { fireGun(); }//start firing missile
		if (fired) animateGun(); //continue firing missile
		if ((EndTurnState==0) ||  (EndTurnState==1))  explode(EndTurnState); //explode a ship
		if (EndTurnState==3) planetDebris();	//draw an explosion on a planet
	}
	
	return 1; 
} 
 
void DrawHud(void) 
{ 
	char textString1[80],textString2[80],textString3[80] ;
	glCullFace(GL_BACK);
	glColor4f(1.0,0.0,0.0,0.3); 
	glBegin(GL_POLYGON);	//POWER BAR 
	glVertex3f(30.0, 25.0,  0.0); 
	glVertex3f(30+(Ships[player].pspeed*125), 25.0,  0.0); 
	glVertex3f(30+(Ships[player].pspeed*125), 26.0, 0.0); 
	glVertex3f(30.0, 26.0, 0.0); 
	glEnd();
	glColor4f(1.0,1.0,1.0,0.2); 
	glBegin(GL_POLYGON);	//POWER BAR UNDERNEATH
	glVertex3f(30.0, 25.0,  0.0); 
	glVertex3f(58, 25.0,  0.0); 
	glVertex3f(58, 26.0, 0.0); 
	glVertex3f(30.0, 26.0, 0.0); 
	glEnd();

	largeFont("POWER", 20, 23, 0.6, 0.0, 0.0);
	if (asknewGame)	largeFont("press n for new game", 16, 43, 0.6, 0.0, 0.0);
	if (player == 0) 
	{
		largeFont("PLAYER 1", 15, 75, 1.0, 0.0, 0.0);
		largeFont("PLAYER 2", 68, 75, 0.3, 0.0, 0.0);
	}
	if (player == 1) 
	{
		largeFont("PLAYER 2", 68, 75, 1.0, 0.0, 0.0);
		largeFont("PLAYER 1", 15, 75, 0.3, 0.0, 0.0);
	}
	sprintf(textString1, "%d :score: %d", p1score, p2score);
	largeFont(textString1, 38, 75, 0.6, 0.0, 0.0);
	//largeFont("CAMERA: ", 64, 25, 0.0, 0.0, 0.7);
	
	//CAMERA INFO
	if (camera==1) largeFont("view: FREE    ", 64, 23, 0.0, 0.0, 0.8); 
	if (camera==2) largeFont("view: PLAYER 1", 64, 23, 0.0, 0.0, 0.8);
	if (camera==3) largeFont("view: PLAYER 2", 64, 23, 0.0, 0.0, 0.8);
	if (camera==4) largeFont("view: MISSILE ", 64, 23, 0.0, 0.0, 0.8);
	if (camera==5) largeFont("view: UNIVERSE ", 64, 23, 0.0, 0.0, 0.8);

	if ((camera==2) || (camera==3)) largeFont("+", 50, 50, 0.0, 0.8, 0.0); //cross hair
	

	int off=8;
	//HELP SCREEN STUFF
	if (help)  
	{
				
		//largeFont("HELLO. THIS IS PLANETS 3D",      35, 68, 1.0, 1.0, 1.0);
		//largeFont("this game is 2 player turn based",      35, 66, 1.0, 1.0, 1.0);
		largeFont("H........exits this help screen", 16, 59+off, 1.0, 1.0, 1.0);
		largeFont("1........Maya camera)", 16, 55+off, 1.0, 1.0, 1.0);
		largeFont("2........player 1 ship point of view ", 16, 51+off, 1.0, 1.0, 1.0);
		largeFont("3........player 2 ship point of view", 16, 47+off, 1.0, 1.0, 1.0);
		largeFont("4........missile camera", 16, 43+off, 1.0, 1.0, 1.0);
		largeFont("5........fixed universe camera", 16, 39+off, 1.0, 1.0, 1.0);
		largeFont("cursors..rotate current player's ship", 16, 35+off, 1.0, 1.0, 1.0);
		largeFont("space....fires missile", 16, 31+off, 1.0, 1.0, 1.0);
		largeFont("escape...exits game", 16, 27+off, 1.0, 1.0, 1.0);
		largeFont("+/-.......UP & DOWN power", 16, 23+off, 1.0, 1.0, 1.0);
		
		//BACKING COLOR
		glColor4f(0.0,0.0,1.0,0.6); 
		glBegin(GL_POLYGON);	
		glVertex3f(0.0, 0.0,  0.0); 
		glVertex3f(100.0,0.0,  0.0); 
		glVertex3f(100.0, 100.0, 0.0); 
		glVertex3f(0.0, 100.0, 0.0); 
		glEnd();		
	

	}

	//WRITE FRAMES PER SECOND
	/*sortFrametime();
	sprintf(textString3, "%f", (1/frametime));
	font(textString3, 20, 75, 0.0, 1.0, 0.0);*/

	glCullFace(GL_FRONT);
 
} 
 
 
void DrawProjectile(void) 
{ 
	glDisable(GL_LIGHTING); 
	glDisable(GL_LIGHT0);
	glCullFace(GL_BACK);
		if (player==0)	glColor4f(1.0,0.0,0.0,0.8);
		if (player==1)	glColor4f(0.0,1.0,0.0,0.8);
		glutSolidSphere(0.02,5,5);
		if (player==0)	glColor4f(1.0,0.0,0.0,0.05);
		if (player==1)	glColor4f(0.0,1.0,0.0,0.05);
		glutSolidSphere(0.035,5,5);
	glCullFace(GL_FRONT);
	glEnable(GL_LIGHTING); 
	glEnable(GL_LIGHT0);
	//glutSolidCube(0.03); 
} 
	 

		 
int StartGame(void) //Called each new game
{ 
 
	float distsqr = 0;
	srand(time(NULL));
	float distsqrd,xdif,ydif,zdif;
	int ok;
 
	//Inititalize Space - star array
	for(int j=0;j<STARS;j++)
		
		{
		space[j].x = (float)(rand() % 500)-250;
		space[j].y = (float)(rand() % 500)-250;
		space[j].z = (float)(rand() % 500)-250;
		}
	
	//Inititalise planets	 
	for(int i=0;i<NUMPLANETS;i++) 
	{ 
		
		if (i==0)
		{
		planets[i].x = (float)((rand() % 1000)/100)-5;
		planets[i].y = (float)((rand() % 1000)/100)-5;
		planets[i].z = (float)((rand() % 1000)/100)-5;
		planets[i].size = (1.0 +   ( (float)  (rand() % (((MAXPLANETSIZE)-1)*1000) ) / 1000 )    ); 
		planets[i].spheres = NULL;
		}

		ok = 0;
		while( ok<i )
			{
			ok=0;
			planets[i].x = (float)((rand() % 1000)/100)-5;
			planets[i].y = (float)((rand() % 1000)/100)-5;
			planets[i].z = (float)((rand() % 1000)/100)-5;
			planets[i].size = (1.0 +   ( (float)  (rand() % (((MAXPLANETSIZE)-1)*1000) ) / 1000 )    ); 
			planets[i].spheres = NULL;

			for( int j=0; j<i; j++ )
				{
				distsqrd =	planets[i].x-planets[j].x * planets[i].x-planets[j].x +
							planets[i].y-planets[j].y * planets[i].y-planets[j].y +
							planets[i].z-planets[j].z * planets[i].z-planets[j].z;
				if( distsqrd < pow( planets[i].size + planets[j].size, 2 ) ) ok++; 
				}
		
			}
	
	}






	//SETUP SPACE BACKDROP
	planets[NUMPLANETS].x =0.0;
	planets[NUMPLANETS].y =0.0;
	planets[NUMPLANETS].z =0.0;
	planets[NUMPLANETS].size = 300.0;
	planets[NUMPLANETS].spheres = NULL;
 
	//Initialize Ships 
	Ships[0].x = -12; 
	Ships[0].y = 0; 
	Ships[0].z = 0; 
	Ships[1].x = 12; 
	Ships[1].y = 0; 
	Ships[1].z = 0; 
	Ships[1].rx = 0; 
	Ships[1].ry = 180; 
	Ships[1].rz = 90; 
 	for(i=0;i<2;i++) 
	{ 
		Ships[i].size = 1.0; 
		Ships[i].px = 1.0; 
		Ships[i].py = 0.0; 
		Ships[i].pz = 0.0; 
		Ships[i].pspeed = 0.12; 
 	} 
 
 
	BuildUniverse();
 
 
return 1; 
} 
 
 
 
//Fire Gun - Positions projectile in front of ship and initialises its velocity vector 
int fireGun(void)	 
{ 
	float HALFPI=PI/180;
	
	//WORK OUT VELOCITY (xyz) THAT PROJECTILE SHOULD START TRAVELLING IN
	Ships[player].px =    ( (cos((HALFPI)*Ships[player].ry)) * (cos((HALFPI)*Ships[player].rx)) ) + Ships[player].x; 
	Ships[player].pz = -1*( (sin((HALFPI)*Ships[player].ry)) * (cos((HALFPI)*Ships[player].rx)) ) + Ships[player].z; 
	Ships[player].py = -1*(  sin((HALFPI)*Ships[player].rx) ) + Ships[0].y; 
	 
	Ships[player].vx =	   Ships[player].pspeed*( (cos((HALFPI)*Ships[player].ry)) * (cos((HALFPI)*Ships[player].rx)) ); 
	Ships[player].vz =  -1*Ships[player].pspeed*( (sin((HALFPI)*Ships[player].ry)) * (cos((HALFPI)*Ships[player].rx)) ); 
	Ships[player].vy =  -1*Ships[player].pspeed*(  sin((HALFPI)*Ships[player].rx) ); 
 
 
	glPushMatrix(); 
		glTranslatef(Ships[player].px, Ships[player].py, Ships[player].pz); 
		DrawProjectile(); 
	glPopMatrix(); 
 
	firing = false; 
	fired  = true; 
			 
		killTail();	 
 		tail = new (aTail); 
		tail->pos.x = Ships[player].px; 
		tail->pos.y = Ships[player].py; 
		tail->pos.z = Ships[player].pz; 
		tail->fade = 1.0; 
		tail->nextPtr = NULL; 
		tail->prevPtr = NULL; 
			 
	return 0; 
 
} 
 
 
//animate gun calculates the path of the projectile through space 
int animateGun(void) 
{ 
	turnlength++;

	float dist,a,b,c,dist2,pathDist; 
	aPoint dif,dif2,dif3,testPos; 
	const float g=0.005; 
	bool missed=false; 
	int numInc; 
	aTail *currentTail,*tempTail; 

 
	glPushMatrix(); 
	 glTranslatef(Ships[player].px, Ships[player].py, Ships[player].pz); 
	 DrawProjectile(); 
	glPopMatrix(); 
 
	//move projectile - increase its position by its velocity - need frametime in here 
	Ships[player].pxold = Ships[player].px; 
	Ships[player].pyold = Ships[player].py; 
	Ships[player].pzold = Ships[player].pz; 
	Ships[player].px += Ships[player].vx; 
	Ships[player].py += Ships[player].vy; 
	Ships[player].pz += Ships[player].vz; 
 




	//CALCULATE TAIL
	currentTail = tail; 
	while(currentTail->nextPtr!=NULL) currentTail=currentTail->nextPtr; 
	tempTail = new (aTail); 
	tempTail->pos.x = Ships[player].px; 
	tempTail->pos.y = Ships[player].py; 
	tempTail->pos.z = Ships[player].pz; 
	tempTail->fade = 1.0; 
	tempTail->nextPtr = NULL; 
	tempTail->prevPtr = currentTail; 
	//currentTail->fade = (currentTail->fade)*0.1; 
	currentTail->nextPtr = tempTail; 
	tempTail = new (aTail); 
	tempTail = tail; 
	while(tempTail->nextPtr!=NULL)  
	{ 
		tempTail->fade = (tempTail->fade)*0.95; 
		tempTail = tempTail->nextPtr; 
	} 
 
	if (tail->fade<0.1) 
	{ 
		tail = tail->nextPtr; 
	} 
  
	currentTail = tail; 
	currentTail = currentTail->nextPtr; 
	 
	glDisable(GL_LIGHTING); 
	glDisable(GL_LIGHT0); 
	glDisable(GL_BLEND);
	while(currentTail->nextPtr!=NULL)		//DRAW TAIL
	{ 
	   glPushMatrix(); 
				glBegin(GL_LINES); 
				if (player == 0)
 					glColor4f(currentTail->nextPtr->fade, 0.0, 0.0, 1.0);
				else
					glColor4f(0.0,currentTail->nextPtr->fade,0.0, 1.0);
				glVertex3f(currentTail->pos.x, currentTail->pos.y, currentTail->pos.z);
				glVertex3f(currentTail->nextPtr->pos.x, currentTail->nextPtr->pos.y, currentTail->nextPtr->pos.z); 
				glEnd(); 
	   glPopMatrix(); 
	   currentTail = currentTail->nextPtr; 
	} 
 	glEnable(GL_BLEND);
	glEnable(GL_LIGHTING); 
	glEnable(GL_LIGHT0); 
	//END CALCULATE TAIL
 


 
 
	//check collisions with enemy ship 
		//SHIP NOTPLAYER TEST 
		dif.x = Ships[player].px - Ships[notplayer].x; 
		dif.y = Ships[player].py - Ships[notplayer].y; 
		dif.z = Ships[player].pz - Ships[notplayer].z; 
		dist =	dif.x*dif.x + dif.y*dif.y + dif.z*dif.z;	 
		if (dist < ((Ships[notplayer].size)*(Ships[notplayer].size)/2) )  
		{ 
			EndTurnState = notplayer; 
			fired=false; 
			notplayer=player; 
			if(player==0) player=1; else player=0;
			if (camera==4) camera=5;
			turnlength=0;
		} 
		//SHIP PLAYER TEST 
		dif.x = Ships[player].px - Ships[player].x; 
		dif.y = Ships[player].py - Ships[player].y; 
		dif.z = Ships[player].pz - Ships[player].z; 
		dist =	dif.x*dif.x + dif.y*dif.y + dif.z*dif.z;	 
		if (dist < ((Ships[player].size)*(Ships[player].size)/2) )  
		{ 
			EndTurnState = player; 
			fired=false; 
			notplayer=player; 
			if(player==0) player=1; else player=0;
			if (camera==4) camera=5;
			turnlength=0;
		} 


		//Too long fired test
		if (turnlength>1000)  //1000 - is the time out length
		{
			EndTurnState = 3; 
			fired=false; 
			notplayer=player; 
			if(player==0) player=1; else player=0;
			if (camera==4) camera=5;
			turnlength=0;
			explodePos.x = Ships[notplayer].px;
			explodePos.y = Ships[notplayer].py;
			explodePos.z = Ships[notplayer].pz;
		}  //END TURN
 
	 
 
 
	//check collisions with planets 
	dif3.x = Ships[player].px - Ships[player].pxold; 
	dif3.y = Ships[player].py - Ships[player].pyold; 
	dif3.z = Ships[player].pz - Ships[player].pzold; 
 
	pathDist = dif3.x*dif3.x + dif3.y*dif3.y + dif3.z*dif3.z; 
	numInc = (int)(pathDist/(TESTSIZE)*(TESTSIZE)); 
	for (int i=0;i<NUMPLANETS;i++) if (fired) 
	{ 
	 for( int j=0; j<=numInc;j++) //TEST ALONG PATH OF PROJECTILE 
	  { 
		  //TEST POINT  THAT IS J LOTS OF INCREMENTS ALONG PATH 
		if (j!=0) testPos.x = Ships[player].pxold + dif3.x/j; else testPos.x = Ships[player].px; 
		if (j!=0) testPos.y = Ships[player].pyold + dif3.y/j; else testPos.y = Ships[player].py; 
		if (j!=0) testPos.z = Ships[player].pzold + dif3.z/j; else testPos.z = Ships[player].pz; 
 
		missed=false; 
 
		dif.x = testPos.x - planets[i].x;		//this sort of thing could be done with operator overloading 
		dif.y = testPos.y - planets[i].y; 
		dif.z = testPos.z - planets[i].z; 
		dist = dif.x*dif.x + dif.y*dif.y + dif.z*dif.z;	 
		 
		if ( dist < ( (planets[i].size)*(planets[i].size) ) )   /* IN A PLANET */ 
 
		{ 
 
		if (planets[i].spheres == NULL) 
			{ 
				planets[i].spheres = new (aSphere); 
				currentSphere = planets[i].spheres; 
				currentSphere->radius = craterSize; 
				currentSphere->position.x = dif.x; 
				currentSphere->position.y = dif.y; 
				currentSphere->position.z = dif.z; 
				currentSphere->nextPtr = NULL; 
				fired=false; /*option 1*/
				explodePos.x = testPos.x;
				explodePos.y = testPos.y;
				explodePos.z = testPos.z;
				notplayer=player; 
				if(player==0) player=1; else player=0;  //END TURN 
				EndTurnState = 3; 
				if (camera==4) camera=5;
				turnlength=0;
			 
			}	 
		 
		   else 
			{ 
				currentSphere = planets[i].spheres; 
				while(currentSphere != NULL && missed==false) 
				{ 
					dif2.x = testPos.x - (currentSphere->position.x+planets[i].x); 
					dif2.y = testPos.y - (currentSphere->position.y+planets[i].y); 
					dif2.z = testPos.z - (currentSphere->position.z+planets[i].z); 
 
				  dist2 = dif2.x*dif2.x + dif2.y*dif2.y + dif2.z*dif2.z; 
					 
				 if ( dist2 > ( (currentSphere->radius)*(currentSphere->radius) ) ) currentSphere = currentSphere->nextPtr; 
				 else 
				 {
					 missed=true; /*option 2*/
					 					 
				 }
				  
				 
				} 
 
				if (missed==false) 
				{	 
					currentSphere = planets[i].spheres; 
					while(currentSphere->nextPtr != NULL) currentSphere = currentSphere->nextPtr; 
					currentSphere->nextPtr = new (aSphere); 
					currentSphere = currentSphere->nextPtr; 
					currentSphere->radius = craterSize; 
					currentSphere->position.x = dif.x; 
					currentSphere->position.y = dif.y; 
					currentSphere->position.z = dif.z; 
					currentSphere->nextPtr = NULL; 
					fired=false; /*option 3*/
					explodePos.x = testPos.x;
					explodePos.y = testPos.y;
					explodePos.z = testPos.z;
					notplayer=player; 
					if(player==0) player=1; else player=0;  //END TURN 
					EndTurnState = 3; 
					if (camera==4) camera=5;
					turnlength=0;
					  
				} 



			} 
			 
 
			//Destroy old vertex list and recreate it from the new imlicit data. 
			if (missed==false) 
			{ 
				planets[i].PlanetObj->DestroyMesh(); 
				planets[i].PlanetObj = new( CImpObject); 
				planets[i].PlanetObj->CreateMesh( (SurfaceFunc) pSurface, i, CUBESIZE ,numcubes, 0.0 , 0.0, 0.0); 
		 	} 
				 
        }					/* END IN A PLANET */ 
 
 
		
			//GRAVITY
			//calculate vector between planet and projetcile 
			a = -1*dif.x; 
			b = -1*dif.y; 
			c = -1*dif.z; 
			//alter projectiles velocity  
			Ships[player].vx += g*a/dist; 
			Ships[player].vy += g*b/dist; 
			Ships[player].vz += g*c/dist; 
		 
 
		//WENT TOO FAR AWAY OPTION
		if (fired)
		if ( (Ships[player].px>20)	  || (Ships[player].py>20)	  ||(Ships[player].pz>20) ||  
			 (Ships[player].px<(-20)) || (Ships[player].py<(-20)) ||(Ships[player].pz<(-20))   ) 
		{ 
			fired = false; 
			EndTurnState = 2; 
			notplayer=player; 
			if(player==0) player=1; 
			else player=0;  
			if (camera==4) camera=5;
			turnlength=0;
		} //END TURN - OPTION 4 -projectile to far away
 
	 
		} 
	} 
 
 
	return 0; 
	 
} 
 
 
void explode(int expstate) 
{ 
 
	float alpha=1.0 - 0.03334*ExplodeTime;
	float half=ExplodeTime/2;
 
	glPushMatrix(); 
				glDisable(GL_LIGHTING); 
				glDisable(GL_LIGHT0);
				glCullFace(GL_BACK);
				glTranslatef(Ships[expstate].x, Ships[expstate].y, Ships[expstate].z); 
				glTranslatef(0.2,0.2,0.2); 
				glColor4f(1.0,0.0,0.0, (alpha)); 
				glutSolidSphere(0.1*half,10.0,10.0); 
				glTranslatef(-0.4,-0.4,-0.4); 
				glColor4f(1.0,0.5,0.0, alpha);
				glutSolidSphere(0.1*half,10.0,10.0); 
				glTranslatef(0.0,0.2,0.4); 
				glColor4f(1.0,0.0,0.0, alpha);
				glutSolidSphere(0.05*half,10.0,10.0);
				glTranslatef(0.0,0.6,-0.6); 
				glColor4f(1.0,0.5,0.0, alpha);
				glutSolidSphere(0.1*half,10.0,10.0);
				glTranslatef(0.0,-0.6,0.6); 
				glColor4f(0.1,0.1,0.1, alpha);
				glutSolidSphere(0.2*half,10.0,10.0);
				glCullFace(GL_FRONT);
				glEnable(GL_LIGHTING); 
				glEnable(GL_LIGHT0);
	glPopMatrix(); 
	ExplodeTime++; 
	if (ExplodeTime>30) 
	{ 
		ExplodeTime=1; 
		EndTurnState=2; 
		asknewGame = true; 
		if(expstate == 1) p1score++;
		if(expstate == 0) p2score++;
	} 
} 

void planetDebris(void)
{
	float alpha=1.0 - 0.03334*ExplodeTime;
	float half=ExplodeTime/2;

	glPushMatrix(); 
				glDisable(GL_LIGHTING); 
				glDisable(GL_LIGHT0);
				glCullFace(GL_BACK);
				glTranslatef(explodePos.x,explodePos.y,explodePos.z); 
				glTranslatef(0.2,0.2,0.2); 
				glColor4f(1.0,0.0,0.0, alpha); 
				glutSolidSphere(0.05*half,5,5); 
				glTranslatef(-0.4,-0.4,-0.4); 
				glColor4f(1.0,0.5,0.0, alpha);
				glutSolidSphere(0.05*half,5,5); 
				glTranslatef(0.0,0.2,0.4); 
				glColor4f(1.0,0.0,0.0, alpha);
				glutSolidSphere(0.05*half,5,5);
				glTranslatef(0.0,0.6,-0.6); 
				glColor4f(1.0,0.5,0.0, alpha);
				glutSolidSphere(0.05*half,5,5);
				glTranslatef(0.0,-0.6,0.6); 
				glColor4f(0.5,0.5,0.5, alpha*0.3);
				glutSolidSphere(0.2*half,5,5);
				glCullFace(GL_FRONT);
				glEnable(GL_LIGHTING); 
				glEnable(GL_LIGHT0);
	glPopMatrix(); 

	ExplodeTime++; 
	if (ExplodeTime>30) 
	{ 
		ExplodeTime=1; 
		EndTurnState=2; 
	} 


}
 
 
 
 
void BuildUniverse(void) 
{ 
	 
	for(int i=0;i<NUMPLANETS;i++) 
	{ 
 	 planets[i].PlanetObj = new( CImpObject); 
	 planets[i].PlanetObj->CreateMesh( (SurfaceFunc) pSurface, i, CUBESIZE, numcubes, 0.0, 0.0, 0.0); 
	} 

	planets[NUMPLANETS].PlanetObj = new( CImpObject); 
	planets[NUMPLANETS].PlanetObj->CreateMesh( (SurfaceFunc) pSurface, NUMPLANETS, 30, 20, 300.0, 0.0, 0.0); 
 
} 
 
 
void killTail(void) 
{ 
 
	while(tail!=NULL) 
	{ 
	 if (tail->prevPtr!=NULL) delete (tail->prevPtr); 
	 tail = tail->nextPtr; 
	} 
 
	delete tail; 
 
} 
 
 
 
void KillUniverse(void) 
{ 
	for(int i=0;i<NUMPLANETS;i++) 
	{ 
 	 planets[i].PlanetObj->DestroyMesh(); 
	} 
	planets[NUMPLANETS].PlanetObj->DestroyMesh(); 

} 
 
 
 
int main(int argc, char** argv) 
{ 
 
 
	CImpObject::Init(); // call to static init function makes cube table 
	
	glutInit(&argc, argv); 
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH); 

	glutGameModeString("1024x768:32@60");
	glutEnterGameMode();

	//glutInitWindowSize (winW, winH); 
	//glutInitWindowPosition (100, 100); 
	//glutCreateWindow ("Planets3D"); 
 
	glutDisplayFunc(display); 
	glutReshapeFunc(reshape); 
	glutKeyboardFunc(keyDown); 
	glutSpecialFunc(specialDown); 
	glutMouseFunc(mouse); 
	glutMotionFunc(motion); 
	glutIdleFunc(idle); 
	glutKeyboardUpFunc(keyUp); 
	glutSpecialUpFunc(specialUp); 
	 
	glutMainLoop(); 
 
	KillUniverse(); 
 
	return 0; 
} 
 
 
